#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <math.h>

/* general program headers */
#include "specimen.h"
#include "patch.h"
#include "gui.h"
#include "bank-builder.h"

static void bankbuilder_init (BankBuilder *bb);

static void bankbuilder_class_init (BankBuilder *bb)
{
	//what needs to go here? 
}

static void bankbuilder_init (BankBuilder *bb)
{
     GtkWidget *label;
     
#ifdef HAVE_GTK2_4 
     /* Setup our file selector */
     bb->file_selector =  gtk_file_chooser_widget_new (GTK_FILE_CHOOSER_ACTION_OPEN);
     gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER(bb->file_selector), 1);
     gtk_box_pack_start(GTK_BOX(bb), bb->file_selector, TRUE, TRUE, SPACING);
     gtk_widget_show(bb->file_selector);     
#endif
     
     bb->hbox = gtk_hbox_new(TRUE, SPACING);
     gtk_box_pack_start(GTK_BOX(bb), bb->hbox, FALSE, FALSE, SPACING);
     gtk_widget_show(bb->hbox);
     
     /*the adjustments and spinners*/
     /* channel */

     label = gtk_label_new("Active Channel:");
     gtk_box_pack_start(GTK_BOX(bb->hbox), label, FALSE, FALSE, 0);
     gtk_widget_show(label);
     
     bb->chan_adj = (GtkAdjustment *) gtk_adjustment_new(1, 1, 16, 1, 0, 0);
     bb->chan_spin = gtk_spin_button_new(bb->chan_adj, 0, 0);
     gtk_box_pack_start(GTK_BOX(bb->hbox), bb->chan_spin, FALSE, FALSE, 0);
     gtk_widget_show(bb->chan_spin);

     /* Range */
     label = gtk_label_new("Note Range:");
     gtk_box_pack_start(GTK_BOX(bb->hbox), label, FALSE, FALSE, 0);
     gtk_widget_show(label);
     
     bb->range_adj = (GtkAdjustment *) gtk_adjustment_new(1, 1, 127, 1, 0, 0);
     bb->range_spin = gtk_spin_button_new(bb->range_adj, 0, 0);
     gtk_box_pack_start(GTK_BOX(bb->hbox), bb->range_spin, FALSE, FALSE, 0);
     gtk_widget_show(bb->range_spin);
}


GType bankbuilder_get_type (void)
{
     static GType bb_type = 0;
     
     if(!bb_type)
     {
	  static const GTypeInfo bb_info =
          {
	       sizeof (BankBuilderClass),
	       NULL, /* base_init */
	       NULL, /* base_finalize */
	       (GClassInitFunc) bankbuilder_class_init,
	       NULL, /* class_finalize */
	       NULL, /* class_data */
	       sizeof (BankBuilder),
	       0,    /* n_preallocs */
	       (GInstanceInitFunc) bankbuilder_init,
	  };
	  bb_type = g_type_register_static (GTK_TYPE_VBOX,
			  "BankBuilder", &bb_info, 0);
     	  if(!bb_type)
	  {
	       errmsg("TYPE INIT FAILED!");
	  }
     }
     return bb_type;
}


GtkWidget* bankbuilder_new (void)
{
     return GTK_WIDGET ( g_object_new (BANK_BUILDER_TYPE, NULL));
}

/* Returns what the current active channel is, so stuff is dropped into the right channel
 * Maybe i should fix this to make it fully OO but i can't be arsed. Stick a static thing
 * at the top and add a return thingy. I'd have to change the
 * whole event handling thing and drag and drop is pretty evil to start with. */
int bb_get_active_channel(void)
{    
     extern BankBuilder *bb;	
     return gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(bb->chan_spin));    
}

int bb_get_range(void)
{
     extern BankBuilder *bb;	
     return gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(bb->range_spin));
}

/* Creat a new patch, with the root note from the keyboard drop and the channel, range from the spinners 
 * It's a list so we can add multiple files from DnD.
 * */ 
void bb_new_patch (int note, GSList *filelist)
{
     int rangenote, i, index;
     int val = -1;
     extern BankBuilder *bb;
     gchar *newfilename = NULL;
     
     /* add all the files that where dropped */
     for(i = 0;i<g_slist_length(filelist);i++)
     {
	  newfilename = g_slist_nth_data (filelist, i);
	  /* Use the filename as the patchname. I think this is the best option for now. */
	  val = patch_create(g_path_get_basename(newfilename));

	  if (val < 0) {
	       errmsg("Failed to create a new patch (%s).\n", patch_strerror(val));
	       return;
	  }
	  /* count from 1 not 0, thus the minus 1 */  
	  patch_set_channel(val, bb_get_active_channel()-1);
	  
	  /* increment channel if multiple files are dropped. */
	  if(g_slist_length(filelist) > 1)
	  {
	       	gtk_spin_button_spin(GTK_SPIN_BUTTON(bb->chan_spin), GTK_SPIN_USER_DEFINED, 1);    
	  }
	  
	  patch_set_note(val, note);
	  patch_sample_load(val, newfilename);	
	  if(bb_get_range() > 1)
	  {
	       debug("setting range");
	       /* Set lower note, only allow odd numbers, round to make sure you always get them
	       * bounds check too... */
	       rangenote = note - floor(bb_get_range() / 2);
	       if(rangenote < 0)
	       {
		    rangenote = 0;
	       }
	       patch_set_lower_note(val, rangenote);
	  
	       rangenote = note + floor(bb_get_range() / 2);
	       if(rangenote > 127)
	       {
		    rangenote = 127;
	       }
	       patch_set_upper_note(val, rangenote);
	       /* tell the patch to listen on a range of notes */
	       patch_set_range(val, 1);
	  }   
     }
     index = patch_get_display_index(val);
     update_interface_helper(index);
     g_free(newfilename);
}
