/* predominantly ripped from the alsa programming tutorial */
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <alsa/asoundlib.h>
#include <glib.h>
#include "specimen.h"
#include "mixer.h"
#include "midi.h"

#ifdef HAVE_LADCCA
#include <ladcca/ladcca.h>
#include "leash.h"
#endif

static sig_atomic_t   running = 0;
static GThread       *midi_thread;

static void midi_action(snd_seq_t *handle) 
{
     snd_seq_event_t *ev;

     do {
	  snd_seq_event_input(handle, &ev);
	  switch (ev->type) {
	  case SND_SEQ_EVENT_CONTROLLER: 
	       debug("Control event on Channel %2d: %5d       \r",
		     ev->data.control.channel, ev->data.control.value);
	       break;
	  case SND_SEQ_EVENT_PITCHBEND:
	       debug("Pitchbender event on Channel %2d: %5d   \r", 
		     ev->data.control.channel, ev->data.control.value);
	       break;
	  case SND_SEQ_EVENT_NOTEON:
               if (ev->data.note.velocity == 0x00)
		    mixer_note_off(ev->data.control.channel, ev->data.note.note);
	       else 
		    mixer_note_on(ev->data.control.channel, ev->data.note.note, ev->data.note.velocity);
	       break;        
	  case SND_SEQ_EVENT_NOTEOFF: 
	       mixer_note_off(ev->data.control.channel, ev->data.note.note);
	       break;        
	  }
	  snd_seq_free_event(ev);
     } while (snd_seq_event_input_pending(handle, 0) > 0);
}

static void *poll_events(void *arg)
{
     snd_seq_t *handle = arg;
     int npfd;
     struct pollfd *pfd;

     npfd = snd_seq_poll_descriptors_count(handle, POLLIN);
     pfd = g_newa(struct pollfd, npfd);
     snd_seq_poll_descriptors(handle, pfd, npfd, POLLIN);

     while (1) {
	  if (poll(pfd, npfd, 100) > 0)
	       midi_action(handle);
	  if (!running)
	       break;
     }

     return 0;
}
     
static int open_seq(snd_seq_t **handle) 
{
     int portid;

     if (snd_seq_open(handle, "default", SND_SEQ_OPEN_INPUT, 0) < 0) {
	  debug("Failed to open ALSA sequencer\n");
	  return MIDI_ERR_SEQ;
     }
     
#ifdef HAVE_LADCCA
     cca_alsa_client_id(leash_get_client(), snd_seq_client_id(*handle));
#endif
    
     snd_seq_set_client_name(*handle, "Specimen Sampler");
     if ((portid = snd_seq_create_simple_port(*handle, "Specimen Sampler",
					      SND_SEQ_PORT_CAP_WRITE|SND_SEQ_PORT_CAP_SUBS_WRITE,
					      SND_SEQ_PORT_TYPE_APPLICATION)) < 0) {
	  debug("Failed to create ALSA sequencer port\n");
	  return MIDI_ERR_PORT;
     }
     
     return 0;
}

int midi_start() 
{
     int err;
     snd_seq_t *handle;

     if (running) {
	  debug("MIDI already running, so not starting\n");
	  return 0;
     }
     
     debug("Starting MIDI\n");
     if ((err = open_seq(&handle)) < 0)
	  return err;

     running = 1;
     midi_thread = g_thread_create(poll_events, (gpointer)handle, TRUE, NULL);
     debug("MIDI started\n");
     return 0;
}

void midi_stop()
{
     if (!running) {
	  debug("MIDI not running, so not stopping\n");
	  return 0;
     }
     
     debug("Stopping MIDI\n");
     running = 0;
     g_thread_join(midi_thread);
     debug("MIDI stopped\n");
}
