#include <sys/time.h>
#include "specimen.h"
#include "driver.h"
#include "ticks.h"

/* gets the current tick (sample count).  N.B., we divide tv_usec by
   1,000,000 to convert to full seconds */
tick_t getticks()
{
     struct timeval time;
     tick_t val;
     static tick_t offset = -1;

     if (offset == -1) {
	  if (gettimeofday(&time, NULL) < 0) {
	       offset = 0;
	  } else {
	       offset = (time.tv_sec + time.tv_usec / 1000000.0);
	  }
     }

     if (gettimeofday(&time, NULL) < 0) {
	  errmsg("Failed to get time\n");
	  return 0;
     }

     val = (time.tv_sec + time.tv_usec / 1000000.0 - offset) * DRIVER->getrate() + DRIVER->getperiodsize();
     return val;
}

/* convert seconds to ticks */
tick_t sec2tick(float secs)
{
     return DRIVER->getrate() * secs;
}

/* convert ticks to seconds */
float tick2sec(tick_t tick)
{
     return tick * 1.0 / DRIVER->getrate();
}

